package fi.jyu.mit.fxgui;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Consumer;

import javafx.beans.property.BooleanProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.property.StringProperty;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.collections.ObservableList;
import javafx.scene.control.ListView;

/**
 * A custom ListView control that supports easy addition of items in Scene
 * Builder.
 * 
 * @author Tero Paavolainen extended from ListChooser
 * @version 13.1.2017
 * @param <T> mitä luokkaa tallennetaan
 *
 */
public class ListChooser<T> extends ListView<StringAndObject<T>> implements Chooser<T> {

	private StringProperty rivit = new SimpleStringProperty("\n");
	private BooleanProperty nollataanko = new SimpleBooleanProperty(false);
	

	/**
	 * Initializes the control.
	 */
	public ListChooser() {
	    super();
	   
	}

	
	/**
	 * Adds the given object with given name
	 * @param name objektin kohdalla näkyvä teksti
	 * @param object tallennettava olio
	 */
	@Override
    public void add(String name, T object){
	    String text = name;
	    if (text != null)
	        text = Functions.removeMnemonic(text);
		getItems().add(new StringAndObject<T>(text, object));
	}
	
	
	/**
	 * Addes the given object to the list
	 * @param object tallennettava olio
	 */
	@Override
    public void add(T object){
		add(null, object);
	}
	
	
	/**
	 * Adds example text to the list
	 * @param text teksti joka näytetään
	 */
	@Override
    public void addExample(String text){
		add(text, null);
	}
	
	
	/**
	 * Clears the ObservableList that holds the objects
	 */
	@Override
    public void clear(){
		getItems().clear();
	}
	
	
    private void setRows(String[] strings) {
        ObservableList<StringAndObject<T>> objects = getItems();
        objects.clear();
        for (String string : strings) {
            add(string, null);
        }
        this.getSelectionModel().selectFirst();
    }
    
    
	/**
	 * Sets the rows.
	 * 
	 * @param jono
	 *            A multiline string, each line representing a single row.
	 */
    @Override
	public void setRivit(String jono) {
		this.rivit.set(jono);
		String[] strings = jono.split("\n"); // erotinMerkki.get());
		setRows(strings);
		//rivit.setAll();
	}
	
	
    @Override
    public void setRivit(String[] rivit) {
        StringBuilder sb = new StringBuilder();
        String sep = "";
        for (String s: rivit) { sb.append(sep).append(s); sep = "\n"; }
        this.rivit.set(sb.toString());
        setRows(rivit);
    }

	/**
	 * Palauttaa valitun olion tai null
	 * @return valittu olio tai null
	 */
	@Override
    public T getSelectedObject() {
        StringAndObject<T> item = getSelectionModel().getSelectedItem();
        if ( item == null ) return null;
	    return item.getObject();
	}
	

	/**
	 * Returns the contents of the ListChooser as a multiline string.
	 * 
	 * @return The contents of the ListChooser as a multiline string.
	 */
	@Override
    public String getRivit() {
		return rivit.get();
	}
	
	
	/**
     * Sets the value if the component should be reset after the original example data
     * @param reset should it be reset
     */
	public void setNollataanko(boolean reset){
		nollataanko.set(reset);
	}
	
	
	 /**
     * Returns if the component was selected to be cleared
     * @return should it be cleared
     */
	public boolean getNollataanko(){
		return nollataanko.get();
	}


	@Override
	public String getSelectedText() {
		StringAndObject<T> item = getSelectionModel().getSelectedItem();
        if ( item == null ) return null;
	    return item.getName();
	}
	
	@Override
    public int getSelectedIndex() {
       return getSelectionModel().getSelectedIndex();
	}

	
    @Override
    public int setSelectedIndex(int index) {
        int oldIndex = getSelectedIndex();
        getSelectionModel().select(index);
        return oldIndex;
    }
	
    
    /**
     * If index already selected and selected = false. Nothing will be selected.
     * Otherwise wont select anything with selected = false;
     */
    @Override
    public int setSelectedIndex(int index, boolean selected){
        if (selected){
           return setSelectedIndex(index); 
        }
        int old  = getSelectedIndex();
        if(index == old){
            setSelectedIndex(-1);
            return old;
        }
        return old;
    }
    

    @Override
    public void addSelectionListener(Consumer<T> event) {
        this.getSelectionModel().selectedItemProperty()
                .addListener(new ChangeListener<StringAndObject<T>>() {
                    @Override
                    public void changed(
                            ObservableValue<? extends StringAndObject<T>> observable,
                            StringAndObject<T> oldValue,
                            StringAndObject<T> newValue) {
                        if (newValue != null)
                            event.accept(newValue.getObject());
                    }
                });

    }
    
    
    @Override 
    public List<T> getObjects() {
        List<T> result = new ArrayList<>();
        for (StringAndObject<T> item: getItems()) result.add(item.getObject());
        return result;
    }

}
